Index: include/asterisk/cdr.h
===================================================================
--- include/asterisk/cdr.h	(revision 218291)
+++ include/asterisk/cdr.h	(working copy)
@@ -119,6 +119,12 @@
 void ast_cdr_free_vars(struct ast_cdr *cdr, int recur);
 int ast_cdr_copy_vars(struct ast_cdr *to_cdr, struct ast_cdr *from_cdr);
 
+/*!
+ * \brief CDR backend callback
+ * \warning CDR backends should NOT attempt to access the channel associated
+ * with a CDR record.  This channel is not guaranteed to exist when the CDR
+ * backend is invoked.
+ */
 typedef int (*ast_cdrbe)(struct ast_cdr *cdr);
 
 /*! \brief Return TRUE if CDR subsystem is enabled */
Index: include/asterisk/channel.h
===================================================================
--- include/asterisk/channel.h	(revision 218291)
+++ include/asterisk/channel.h	(working copy)
@@ -562,6 +562,8 @@
 	 *  bridge terminates, this will allow the hangup in the pbx loop to be run instead.
 	 *  */
 	AST_FLAG_BRIDGE_HANGUP_DONT = (1 << 18),
+	/*! This flag indicates whether the channel is in the channel list or not. */
+	AST_FLAG_IN_CHANNEL_LIST = (1 << 19),
 };
 
 /*! \brief ast_bridge_config flags */
Index: main/channel.c
===================================================================
--- main/channel.c	(revision 218291)
+++ main/channel.c	(working copy)
@@ -924,6 +924,8 @@
 
 	tmp->tech = &null_tech;
 
+	ast_set_flag(tmp, AST_FLAG_IN_CHANNEL_LIST);
+
 	AST_RWLIST_WRLOCK(&channels);
 	AST_RWLIST_INSERT_HEAD(&channels, tmp, chan_list);
 	AST_RWLIST_UNLOCK(&channels);
@@ -1300,18 +1302,25 @@
 	struct varshead *headp;
 	struct ast_datastore *datastore = NULL;
 	char name[AST_CHANNEL_NAME], *dashptr;
+	int inlist;
 	
 	headp=&chan->varshead;
+
+	ast_channel_lock(chan);
+	inlist = ast_test_flag(chan, AST_FLAG_IN_CHANNEL_LIST);
+	ast_channel_unlock(chan);
 	
-	AST_RWLIST_WRLOCK(&channels);
-	if (!AST_RWLIST_REMOVE(&channels, chan, chan_list)) {
-		AST_RWLIST_UNLOCK(&channels);
-		ast_log(LOG_ERROR, "Unable to find channel in list to free. Assuming it has already been done.\n");
+	if (inlist) {
+		AST_RWLIST_WRLOCK(&channels);
+		if (!AST_RWLIST_REMOVE(&channels, chan, chan_list)) {
+			AST_RWLIST_UNLOCK(&channels);
+			ast_debug(1, "Unable to find channel in list to free. Assuming it has already been done.\n");
+		}
+		/* Lock and unlock the channel just to be sure nobody has it locked still
+		   due to a reference retrieved from the channel list. */
+		ast_channel_lock(chan);
+		ast_channel_unlock(chan);
 	}
-	/* Lock and unlock the channel just to be sure nobody has it locked still
-	   due to a reference retrieved from the channel list. */
-	ast_channel_lock(chan);
-	ast_channel_unlock(chan);
 
 	/* Get rid of each of the data stores on the channel */
 	while ((datastore = AST_LIST_REMOVE_HEAD(&chan->datastores, entry)))
@@ -1390,7 +1399,8 @@
 
 	ast_string_field_free_memory(chan);
 	ast_free(chan);
-	AST_RWLIST_UNLOCK(&channels);
+	if (inlist)
+		AST_RWLIST_UNLOCK(&channels);
 
 	/* Queue an unknown state, because, while we know that this particular
 	 * instance is dead, we don't know the state of all other possible
@@ -1613,6 +1623,16 @@
 		ast_channel_unlock(chan);
 		return 0;
 	}
+	ast_channel_unlock(chan);
+
+	AST_RWLIST_WRLOCK(&channels);
+	if (!AST_LIST_REMOVE(&channels, chan, chan_list)) {
+		ast_log(LOG_ERROR, "Unable to find channel in list to free. Assuming it has already been done.\n");
+	}
+	AST_RWLIST_UNLOCK(&channels);
+
+	ast_channel_lock(chan);
+	ast_clear_flag(chan, AST_FLAG_IN_CHANNEL_LIST);
 	free_translation(chan);
 	/* Close audio stream */
 	if (chan->stream) {
